/* slld_hal.h - HAL Header file for Cypress SPI-flash Low Level Driver */

/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#ifndef __INC_lld_halh
#define __INC_lld_halh

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

// The following are specific to the Xilinx Zynq-7000 Quad SPI controller

#define QSPI_IO_REGS            ((volatile BYTE *)0xE000D000)
#define SPI_CONFIG_REG          (*(volatile DWORD *)(QSPI_IO_REGS))
#define SPI_INTRPT_STATUS_REG   (*(volatile DWORD *)(QSPI_IO_REGS + 0x04))
#define SPI_INTRPT_EN_REG       (*(volatile DWORD *)(QSPI_IO_REGS + 0x08))
#define SPI_INTRPT_DIS_REG      (*(volatile DWORD *)(QSPI_IO_REGS + 0x0C))
#define SPI_EN_REG              (*(volatile DWORD *)(QSPI_IO_REGS + 0x14))
#define SPI_TXD0                (*(volatile DWORD *)(QSPI_IO_REGS + 0x1C))  // TX FIFO, 4 bytes
#define SPI_RX_DATA_REG         (*(volatile DWORD *)(QSPI_IO_REGS + 0x20))
#define SPI_GPIO                (*(volatile DWORD *)(QSPI_IO_REGS + 0x30))
#define SPI_TXD1                (*(volatile DWORD *)(QSPI_IO_REGS + 0x80))  // TX FIFO, 1 byte
#define SPI_TXD2                (*(volatile DWORD *)(QSPI_IO_REGS + 0x84))  // TX FIFO, 2 bytes
#define SPI_TXD3                (*(volatile DWORD *)(QSPI_IO_REGS + 0x88))  // TX FIFO, 3 bytes
#define SPI_LQSPI_CFG           (*(volatile DWORD *)(QSPI_IO_REGS + 0xA0))
#define SPI_MOD_ID              (*(volatile DWORD *)(QSPI_IO_REGS + 0xFC))

#define SPI_CONFIG_REG_IFMODE_MASK          0x80000000  // 0 = legacy, 1 = dual/quad mode
#define SPI_CONFIG_REG_ENDIAN_MASK          0x04000000  // little/big endian writing TX register and reading RX register
#define SPI_CONFIG_REG_MANSTRT_MASK         0x00010000  // manual start command, 1 = start transmission of data
#define SPI_CONFIG_REG_MANSTRTEN_MASK       0x00008000  // enable/disable manual start command
#define SPI_CONFIG_REG_SSFORCE_MASK         0x00004000  // manual/automatic handling of CS#
#define SPI_CONFIG_REG_PCS_MASK             0x00000400  // drives peripheral CS# if SSFORCE is set to manual
#define SPI_CONFIG_REG_FIFO_WIDTH_MASK      0x000000C0  // FIFO bus width 11 = 32b, 10 = 24b, 01 = 16b, 00 = 8b
#define SPI_CONFIG_REG_BAUD_RATE_DIV_MASK   0x00000038  // clock frequency master mode baud rate divisor
#define SPI_CONFIG_REG_BAUD_RATE_DIV_4      0x00000008  // clock frequency master mode baud rate divide by 4 (001b)
#define SPI_CONFIG_REG_BAUD_RATE_DIV_8      0x00000010  // clock frequency master mode baud rate divide by 8 (010b)
#define SPI_CONFIG_REG_CPHA_MASK            0x00000004  // clock phase (CPHA)
#define SPI_CONFIG_REG_CPOL_MASK            0x00000002  // clock polarity (CPOL)
#define SPI_CONFIG_REG_MSTREN_MASK          0x00000001  // 1 = SPI in master mode, 0 = SPI in slave mode
#define SPI_INTRPT_STATUS_REG_RXNEMPTY_MASK 0x00000010  // 1 = RX FIFO is not empty (>= threshold)
#define SPI_INTRPT_STATUS_REG_TXFULL_MASK   0x00000008  // 1 = TX FIFO is full
#define SPI_INTRPT_STATUS_REG_TXOW_MASK     0x00000004  // 1 = TX FIFO is not full
#define SPI_INTRPT_STATUS_REG_RXOVR_MASK    0x00000001  // 1 = RX FIFO had an overflow
#define SPI_INTRPT_ALL_VALUE                0x000000FF  // TX FIFO and RX FIFO interrupt flags
#define SPI_EN_REG_ENABLE_VALUE             0x00000001
#define SPI_EN_REG_DISABLE_VALUE            0x00000000
#define SPI_GPIO_WP_N_MASK                  0x00000001
#define SPI_LQSPI_CFG_DISABLE_VALUE         0x00000000  // disable linear mode and use single SPI device
#define SPI_LQSPI_CFG_DUAL_FLASH_VALUE      0x600001BB  // disable linear mode and use dual SPI devices
#define SPI_MOD_ID_VALUE                    0x01090101L // module identification register expected value

#define SPI_INTRPT_MAX_POLLS                100000
#define CONTROLLER_BUFFER_SIZE              252         // used in slld.c
#define SPI_FIFO_SIZE_IN_BYTES              252         // used in slld_hal_zynq7000.c
#define SPI_CONTROLLER_WORD_SIZE            4           // 4 bytes in a word

#define SLCR                    ((volatile BYTE *)0xF8000000)   // system level control registers
#define SLCR_LQSPI_CLK_CTRL_REG (*(volatile DWORD *)(SLCR + 0x14C))
#define SLCR_MIO_PIN_00_OFFSET  0x700
#define SLCR_QSPI_RST_CTRL_REG  (*(volatile DWORD *)(SLCR + 0x230))
#define MIO_START_PIN_NUMBER_SINGLE_QSPI    1
#define MIO_END_PIN_NUMBER_SINGLE_QSPI      6
#define MIO_START_PIN_NUMBER_DUAL_QSPI      0
#define MIO_END_PIN_NUMBER_DUAL_QSPI        13
#define MIO_PIN_QSPI_3V_NO_PULLUP_VALUE     0x00000702L // expected value for each MIO pin
#define QSPI_REF_RST_VALUE                  0x00000002  // 1 = reset APB interface, QSPI controller protocol engine
#define LQSPI_CPU1X_RST_VALUE               0x00000001  // 1 = reset APB interface, TX/RX FIFOs, QSPI controller regs
#define QSPI_CLK_ARM_PLL_DIV7_VALUE         0x00000721L // divide ARM PLL by 7 for 200MHz QSPI controller clock

#define GPIO_REGS               ((volatile BYTE *)0xE000A000)
#define GPIO_MASK_DATA0_LSW_REG (*(volatile DWORD *)(GPIO_REGS))
#define GPIO_MIO7_OUTPUT_1      0xFF7F0080
#define GPIO_MIO7_OUTPUT_0      0xFF7F0000


// Reorder and shift address bytes into the upper 3 bytes of a DWORD for the TXD0 register
// Starts in a DWORD as 0x??332211, where 33 is the MSB, 22 is the middle byte, and 11 is the LSB.
// Ends in a DWORD as 0x112233??, where the lowest byte will be the command/opcode.
#define ADDR_UPPER_3B   (((sys_addr & 0x00FF0000) >> 8) | ((sys_addr & 0x0000FF00) << 8) | ((sys_addr & 0x000000FF) << 24))

// For a 4 byte address, reorder and shift the 3 MSB of address into the upper 3 bytes of a DWORD
// for the TXD0 register.  Starts in a DWORD as 0x44332211, where 44 is the MSB, 33 is next,
// followed by 22, and 11 is the LSB.  Ends in one DWORD as 0x223344??, where the lowest byte will be
// the command/opcode, and a second DWORD as 0x??????11, where the LSB of address is in the lowest byte.
#define ADDR_UPPER_4B   (((sys_addr & 0xFF000000) >> 16) | (sys_addr & 0x00FF0000) | ((sys_addr & 0x0000FF00) << 16))
#define ADDR_LOWER_4B   (sys_addr & 0x000000FF)

// Number of bits to shift, converted from byte offset into bit offset
#define SHIFT           (data_offset * 8)

/* public function prototypes */

// HAL initialization function
SLLD_STATUS FLASH_INIT
(
void
);

// HAL read functions API
SLLD_STATUS FLASH_READ
(
BYTE    device_num,                     /* device number */
BYTE    command,                                    /* write a single command byte to flash */
ADDRESS sys_addr,                                   /* system address to be used */
BYTE   *data_buffer,                                /* Pointer to the data buffer where to store the read data */
int     Number_Of_Read_Bytes                        /* number of bytes to be read */
);

// HAL write functions API
SLLD_STATUS FLASH_WRITE
(
BYTE    device_num,                     /* device number */
BYTE    command,                                    /* write a single command byte to flash */
ADDRESS sys_addr,                                   /* system address to be used */
BYTE   *data_buffer,                                /* Pointer to the data buffer containing data to be written */
int     Number_Of_Written_Bytes                     /* number of bytes to be written */
);

// HAL polling function API
DWORD FLASH_POLL
(
void
);

void FLASH_EMPTY_TXFIFO
(
void
);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INC_lld_halh */

