/* slld_hal.h - HAL Header file for Cypress SPI-flash Low Level Driver */

/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#ifndef __INC_lld_halh
#define __INC_lld_halh

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <fcntl.h>
#include <sys/ioctl.h>

typedef struct 
{
    unsigned int Has_Cmd:1;
    unsigned int CmdMode:4; //4
    unsigned int Res:2;//6:5
    unsigned int Has_Addr:1;//7
    unsigned int AddrMode:4;//11:8
    unsigned int Addr_Len:2;//13:12
    unsigned int Has_ModeBit:1;//14
    unsigned int Has_Data:1;//15
    unsigned int DataMode:4;//19:16
    unsigned int DummyCycCnt:8;
    unsigned int RdWr:1;
    unsigned int Res1:2;
    unsigned int Has_Dlp:1;
} SubCommand;

typedef struct 
{
    int Cmd;
    union {
        SubCommand SubCmd;
        unsigned int SubCmdVal;
    };
} Instruction;

//***************************************************************************//
// Instruction Set in RAM Table
// Command, Address, Modebit, Dummy, Data, R/W, Val
//
// Command:     2 words Address Offset in RAM
// Address:     Byte Count: 3 bits, Mode: 3 bits
// ModeBit:     Byte Count: 3 bits, Mode: 3 bits
// Dummy:       SCK Cycle Count: 8 bits, Mode: 3 bits=0
// Data:        En of Data: 1 bits (1--with data, 0-- no data), Mode: 3 bits
// R/W:         Read or Write: 1 bit (1-- Read,  0-- Write)
// Val:         valid: 1 bit (1-- valid,  0-- invalid)
//
// Note: Mode   0 --normal read mode
//              1 --dual read mode
//              2 --quad read mode
//              3 --parrallel read mode
//              4 --Fast DDR
//              5 --Dual DDR
//              6 --Quad DDR
//              7 --Read DDR
//***************************************************************************//
enum
{
    CMD_NO_ =   0, //don't have dlp
    CMD_YES =   1, //have dlp

    CMD_SDR_SINGLE  =   0,//sdr single
    CMD_SDR_DUAL    =   1,//sdr dual
    CMD_SDR_QUAD    =   2,//sdr quad
    CMD_DDR_SINGLE  =   4,//ddr single
    CMD_DDR_DUAL    =   5,//ddr dual
    CMD_DDR_QUAD    =   6,//ddr quad
    CMD_DUALQUAD_SDR_SINGLE =   8,//dual-quad sdr single
    CMD_DUALQUAD_SDR_DUAL   =   9,//dual-quad sdr  dual
    CMD_DUALQUAD_SDR_QUAD   =   10,//dual-quad sdr  quad
    CMD_DUALQUAD_DDR_SINGLE =   12,//dual-quad ddr single
    CMD_DUALQUAD_DDR_DUAL   =   13,//dual-quad ddr  dual
    CMD_DUALQUAD_DDR_QUAD   =   14,//dual-quad ddr  quad

    ADDR_NO_    =   0,    // no addr
    ADDR_YES    =   1,    // no addr

    ADD_SDR_SINGLE  =   0,//sdr single
    ADD_SDR_DUAL    =   1,//sdr dual
    ADD_SDR_QUAD    =   2,//sdr quad
    ADD_DDR_SINGLE  =   4,//ddr single
    ADD_DDR_DUAL    =   5,//ddr dual
    ADD_DDR_QUAD    =   6,//ddr quad
    ADD_DUALQUAD_SDR_SINGLE =   8,//dual-quad sdr single
    ADD_DUALQUAD_SDR_DUAL   =   9,//dual-quad sdr  dual
    ADD_DUALQUAD_SDR_QUAD   =   10,//dual-quad sdr  quad
    ADD_DUALQUAD_DDR_SINGLE =   12,//dual-quad ddr single
    ADD_DUALQUAD_DDR_DUAL   =   13,//dual-quad ddr  dual
    ADD_DUALQUAD_DDR_QUAD   =   14,//dual-quad ddr  quad

    ADDR_8BIT  = 0,    // 1 address byte
    ADDR_16BIT  = 1,    // 2 address byte
    ADDR_24BIT  = 2,    // 3 address byte
    ADDR_32BIT  = 3,    // 4 address byte

    MODEBIT_NO_ =   0,    // no MODEBIT
    MODEBIT_YES =   1,    // has MODEBIT

    DATA_NO_    = 0,    // No data
    DATA_YES    = 1,    // With data

    DATA_SDR_SINGLE =   0,//sdr single
    DATA_SDR_DUAL   =   1,//sdr dual
    DATA_SDR_QUAD   =   2,//sdr quad
    DATA_DDR_SINGLE =   4,//ddr single
    DATA_DDR_DUAL   =   5,//ddr dual
    DATA_DDR_QUAD   =   6,//ddr quad
    DATA_DUALQUAD_SDR_SINGLE    =   8,//dual-quad sdr single
    DATA_DUALQUAD_SDR_DUAL  =   9,//dual-quad sdr  dual
    DATA_DUALQUAD_SDR_QUAD  =   10,//dual-quad sdr  quad
    DATA_DUALQUAD_DDR_SINGLE    =   12,//dual-quad ddr single
    DATA_DUALQUAD_DDR_DUAL  =   13,//dual-quad ddr  dual
    DATA_DUALQUAD_DDR_QUAD  =   14,//dual-quad ddr  quad

    OP_WR       = 0,    // R/W:Write
    OP_RD       = 1,     // R/W:Read

    DLP_NO_ =   0, //don't have dlp
    DLP_YES =   1 //have dlp
};

enum SPIMC_REGISTER
{
	// add by Nathan, for zynq QSPI controller register
	SPIMC_REG_VER = 0,
	SPIMC_REG_START  = 0x4,
	SPIMC_REG_GLOBAL_CTRL = 0x8,
	SPIMC_REG_STATUS = 0xC,
	SPIMC_REG_FREQ = 0x10,
	SPIMC_REG_WR_DATA = 0x14,
	SPIMC_REG_RD_DATA = 0x18,
	SPIMC_REG_WR_FIFO_PROG_THRD = 0x1C,
	SPIMC_REG_RD_FIFO_PROG_THRD = 0x20,
	SPIMC_REG_INTERRUPT_STATUS = 0x24,
	SPIMC_REG_INTERRUPT_ENABLE = 0x28,
	SPIMC_REG_DLP_PAT = 0x2C,
	SPIMC_REG_RX_DLP_VALUE1 = 0x30,
	SPIMC_REG_RX_DLP_VALUE2 = 0x34,
	SPIMC_REG_RX_SET_SEL1 = 0x38,
	SPIMC_REG_RX_SET_SEL2 = 0x3C,
	SPIMC_REG_DAM_CMD = 0x40,
	SPIMC_REG_DAM_HIGH_ADDR = 0x44,
	SPIMC_REG_SW_RESET = 0x48,
	SPIMC_REG_BIST_FREQ = 0x4C,
	SPIMC_REG_BIST_WR_DATA = 0x50,
	SPIMC_REG_BIST_RD_DATA = 0x54,
	SPIMC_REG_DEBUG = 0x58,
	SPIMC_REG_CMD_CONFIG_VALID = 0x5C,
	SPIMC_REG_CONFIG_CTRL = 0x60,
	SPIMC_REG_CMD = 0x64,
	SPIMC_REG_ADDRESS = 0x68,
	SPIMC_REG_DATA_LEN = 0x6C,
	SPIMC_REG_MODEBIT = 0x70,
	SPIMC_REG_HOLD_START = 0x74,
	SPIMC_REG_HOLD_LEN = 0x78,
	SPIMC_REG_CMD_CONFIG_INFO = 0x7C,
	SPIMC_REG_CS_CLK_DELAY_CTRL = 0x80,
	SPIMC_REG_SPI_CLK_CYCLES = 0x84,
	SPIMC_REG_EXPECTED_REG_VALUE = 0x88,
	SPIMC_REG_EXPECTED_REG_VALUE_EN = 0x8C,
	SPIMC_REG_BIST_CTRL = 0x90

};

enum
{
    NO_CMD   = 0xFFFF,
    NO_ADDR = 0xFFFFFFFF,
    NO_MB   = 0xFFFF
};


// should be aligned with os_ioctl.h
// for SLLD testing on ZYNQ, reuse SLT driver functions
enum{
    API_SLLD_DEV_INIT,
    API_SLLD_DEV_REG_READ,
    API_SLLD_DEV_REG_WRITE,
    API_SLLD_DEV_CMD_READ,
    API_SLLD_DEV_CMD_WRITE,
    API_SLLD_DEV_DEINIT
};
#define IOC_SLLD_OP         'L'

// argument for IOCTL
typedef struct osarg_struct
{
    DWORD   dwArg1;     // input argument 1
    DWORD   dwArg2;     // input argument 2
    DWORD   dwArg3;     // input argument 3
    DWORD   dwArg4;     // input argument 4
    DWORD   dwArg5;     // input argument 5
    DWORD   dwResult;   // output result
} OSARGS, *POSARGS;

#define IOCNR_SLLD_DEV_INIT                              _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_INIT, sizeof(OSARGS))
#define IOCNR_SLLD_DEV_REG_READ                                _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_REG_READ, sizeof(OSARGS))
#define IOCNR_SLLD_DEV_REG_WRITE                                 _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_REG_WRITE, sizeof(OSARGS))
#define IOCNR_SLLD_DEV_CMD_READ                                 _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_CMD_READ, sizeof(OSARGS))
#define IOCNR_SLLD_DEV_CMD_WRITE                                 _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_CMD_WRITE, sizeof(OSARGS))
#define IOCNR_SLLD_DEINIT                                 _IOC(_IOC_READ|_IOC_WRITE,IOC_SLLD_OP, API_SLLD_DEV_DEINIT, sizeof(OSARGS))

// the following are for FLL devices
enum{
    FLL_128_L = 1,
    FLL_256_L = 2,
    FLL_512_L = 3,
    FLL_512_DDP = 4,
    FLL_256_DQ = 5,
    FLL_512_DQ = 6
};


// command op code definiation, don't need here, the command code is in slld.h
// the following should be aligned with drv_fll.h in driver
enum
{
    Reserved00,                 WriteRegister,              PageProgram,                SlowRead,
    WriteDisable,               ReadStatusRegister1,        WriteEnable,                ReadStatusRegister2,
    Reserved08,                 Reserved09,                 Reserved0A,                 FastRead,
    FastRead4,                  Reserved0D,                 Reserved0E,                 Reserved0F,
    Reserved10,                 Reserved11,                 PageProgram4,               SlowRead4,
    Reserved14,                 ReadConfigurationRegister2,                 Reserved16,                 Reserved17,
    Reserved18,                   Reserved19,                    Reserved1A,                 Reserved1B,
    Reserved1C,                 Reserved1D,                 Reserved1E,                 Reserved1F,
    SectorErase,            SectorErase4,   Reserved22,                 Reserved23, // Modify Parameter4KBsectorErase.4 to sectorErase. 4, by Nathan
    Reserved24,                 Reserved25,                 Reserved26,                 Reserved27,
    Reserved28,                 Reserved29,                 Reserved2A,                 ReadIRPRegister,
    Reserved2C,                 Reserved2D,                 Reserved2E,                 ProgramIRPRegister,
    ClearStatusRegister,        Reserved31,                 QuadPageProgram,                 ReadConfigurationRegister3,
    QuadPageProgram4,                 ReadConfigurationRegister1, LockIBL3,                 Reserved37, //Modified Reserved36 to LockIBL3, by Nathan
    EnterQPI,                 UnLockIBL3,                 Reserved3A,                 ReadDualOut,//Modify Reserved3B to ReadDualOut, Reserved39 to UnLockIBL3 by Nathan
    ReadDualOut4,                 ReadIBL3,                 Reserved3E,                 Reserved3F,////Modify Reserved3B to ReadDualOut4, Reserved3D to ReadIBL3 by Nathan
    Reserved40,    ReadDataLearningRegister,    ProgramSecurityRegion,                 ProgramNVDataLearningRegister, //Modify Parameter8KBsectorErase to Reserved40, by Nathan, OTPProgram to Reserved42
    EraseSecurityRegion,                 Reserved45,                 Reserved46,                 Reserved47,
    ReadSecurityRegion,                 Reserved49,                 WriteVDataLearningRegister,         ReadUniqueID,//Modify Reserved48 to ReadSecurityRegion, by Nathan, OTPRead to Reserved4B
    Reserved4C,                 Reserved4D,                 Reserved4E,                 Reserved4F,
    WriteEnableVolatileStatusRegister,                 Reserved51,                 HalfBlockErase,                 HalfBlockErase4, //Modify Reserved52.53 to HalfBlockErase.4, Reserved50  to WriteEnableVolatileStatusRegister by Nathan
    Reserved54,                 Reserved55,                 Reserved56,                 Reserved57,
    Reserved58,                 Reserved59,                 ReadSFDPRegister,           Reserved5B,
    Reserved5C,                 Reserved5D,                 Reserved5E,                 Reserved5F,
    BulkErase,                  Reserved61,                 Reserved62,                 Reserved63,
    Reserved64,                 ReadAnyRegister,            SoftwareResetEnable,        Reserved67,
    Reserved68,                 Reserved69,                 Reserved6A,                 ReadQuadOut,//Modify Reserved6C to ReadQuadOut, by Nathan
    ReadQuadOut4,                 Reserved6D,                 Reserved6E,                 Reserved6F,//Modify Reserved6C to ReadQuadOut4, by Nathan
    Reserved70,                 WriteAnyRegister,           Reserved72,                 Reserved73,
    Reserved74,                 EraseSuspend,               Reserved76,                 SetBurstLength,//Modify Reserved77 to SetBurstLength, by Nathan
    Reserved78,                 Reserved79,                 EraseResume,                Reserved7B,
    Reserved7C,                 Reserved7D,                 LockGlobalIBL,                 Reserved7F, //Modified Reserved7E to LockGlobalLBL
    Reserved80,                 Reserved81,                 Reserved82,           Reserved83,
    Reserved84,                 Reserved85,             Reserved86,                 Reserved87,//Modify  Programsuspend to Reserved85, by Nathan
    Reserved88,                 Reserved89,                 Reserved8A,              Reserved8B,//Modify  ProgramResume to Reserved8A, by Nathan
    Reserved8C,                 Reserved8D,                 Reserved8E,                 Reserved8F,
    Reserved90,                 Reserved91,                 Reserved92,                 Reserved93,
    Reserved94,                 Reserved95,                 Reserved96,                 Reserved97,
    UnLockGlobalIBL,                 SoftwareReset,              Reserved9A,                 Reserved9B, //Modifed Reserved98 to UnlockGlobalIBL, by Nathan
    Reserved9C,                 Reserved9D,                 Reserved9E,                 ReadIdentification,
    ReservedA0,                 ReservedA1,                 ReservedA2,                 ReservedA3,
    ReservedA4,                 ReservedA5,                 LockProtectionRegister,            ReadProtectionRegister, //Modified PPBLockBitRead to ReadProtectionRegister, by Nathan
    ReservedA8,                 ReservedA9,                 ReservedAA,                 ReleaseFromDeepPowerDown,
    ReservedAC,                 ReservedAD,                 ReservedAE,                 ReadQuadID,
    ReservedB0,        ReservedB1,                 ReservedB2,                 ReservedB3,
    ReservedB4,                 ReservedB5,                 ReservedB6,                 Enter4ByteAddr,
    ReservedB8,                 EnterDeepPowerDown,                 ReservedBA,                 DualIORead,
    DualIORead4,                ReservedBD,                 ReservedBE,                 ReservedBF,
    ReservedC0,             ReservedC1,                 ReservedC2,                 ReservedC3, //Modify SetBurstLength to ReservedC0, by Nathan
    ReservedC4,                 ReservedC5,                 ReservedC6,                 BulkEraseA,
    ReservedC8,                 ReservedC9,                 ReservedCA,                 ReservedCB,
    ReservedCC,                 ReservedCD,                 ReservedCE,                 ReservedCF,
    ReservedD0,        ReservedD1,                 ReservedD2,                 ReservedD3,
    ReservedD4,                 ReservedD5,                 ReservedD6,                 ReservedD7,
    BlockErase,                ReservedD9,                 ReservedDA,                 ReservedDB, //Modify SectorErase to BlockErase, by Nathan
    BlockErase4,               ReservedDD,                 ReservedDE,                 ReservedDF,  //Modify SectorErase4 to BlockErase4, by Nathan
    ReadIBL4,                   LockIBL4,                   UnLockIBL4,                   SetPointerRegionProtection4, //Modify DYBRead4 to ReadIBL4, DYBWrite4 to LockIBL4, PPBRead4 to UnLockIBL4  by Nathan 7/2/2015 PPBProgram4 to SetPointerRegionProtect4 7/15/2015
    ReservedE4,                   ReservedE5,                 ReservedE6,                 PasswordRead,
    PasswordProgram,            Leave4ByteAddr,             UnlockPassword,                 QuadIORead,//Modify PasswordUnlock to Leave4ByteAddr, EA to PasswordUnlock by Nathan
    QuadIORead4,                DDRQuadIORead,              DDRQuadIORead4,             ReservedEF,
    ReservedF0,        ReservedF1,                 ReservedF2,                 ReservedF3,
    ReservedF4,                 ExitQPI,                 ReservedF6,                 ReservedF7,
    ReservedF8,                 ReservedF9,                 ReservedFA,                    SetPointerRegionProtection,
    ReservedFC,                    ReservedFD,                 ReservedFE,                 ModeBitReset //Modify PPBProgram to SetPointerRegionProtect  by Nathan 7/15/2015
};

// the following are copied from slt library. list them just in case we may use slt api
// it should be aligned with api_wrapper.h
enum API
{
    API_SysSetGlobalVar                     = 0x00,
    API_SysGetGlobalVar,
    API_SysFindDevice                       = 0x08,
    API_SysCloseDevice,
    API_SysGetErrorText,
    API_SysCheckUserBreak,
    API_SysShowDriverInformation,
    API_SysWaitUSec                         = 0x10,
    API_SysFormatDeltaTime,
    API_SysGetDeltaTime,
    API_SysGetTimestamp,
    API_SysSetLog                           = 0x20,
    API_SysClearLog,
    API_SysOutputLog,
    API_SysGetLog,

    API_DrvGetDeviceInfo                    = 0x00,
    API_DrvGetDeviceGeometry,
    API_DrvIncreaseStatusPollingTime        = 0x10,
    API_DrvSetSectorEraseTimeout,
    API_DrvSetChipEraseTimeout,
    API_DrvSetWordProgramTimeout,
    API_DrvSetBufferProgramTimeout,
    API_DrvSetEraseSuspendTimeout,
    API_DrvSetProgramSuspendTimeout,
    API_DrvSetPPBLockUnlockTimeout,
    API_DrvSetCRCSuspendTimeout,
    API_DrvSetEvaluateErasStatusTimeout,
    API_DrvGetBadSector,
    API_DrvSetBadSector,
    API_DrvClearBadSector,
    API_DrvClearAllBadSectors,

    API_DrvSetAddressMode                   = 0x00,
    API_DrvSetReadCommandMode,
    API_DrvSetProgramCommandMode,
    API_DrvSetClock                         = 0x08,
    API_DrvSetClockNotLC,
    API_DrvGetClock,
    API_DrvSetWP,
    API_DrvSetupHold,
    API_DrvEnableHold,
    API_DrvDisableHold,
    API_DrvWriteOpcode                      = 0x20,
    API_DrvWriteAddress,
    API_DrvWriteModeBit,
    API_DrvWriteCount,
    API_DrvReadDataBit,
    API_DrvWriteDataBit,
    API_DrvReadControllerRegister           = 0x40,
    API_DrvWriteControllerRegister,
    API_DrvResetController,
    API_DrvGetDevicePowerStatus             = 0x60,
    API_DrvPowerOnDevice,
    API_DrvPowerOffDevice,

    API_DrvReadID                           = 0x00,
    API_DrvReadSFDP,
    API_DrvReadUniqueID,
    API_DrvGetDeviceStatus                  = 0x08,
    API_DrvReadStatusRegister,
    API_DrvWriteStatusRegister,
    API_DrvClearDeviceStatus,
    API_DrvClearDeviceStatusAlternate,
    API_DrvReadConfigurationRegister,
    API_DrvWriteConfigurationRegister,
    API_DrvReadNVRegister,
    API_DrvReadVRegister,
    API_DrvWriteNVRegister,
    API_DrvWriteVRegister,
    API_DrvEnableQuad,
    API_DrvDisableQuad,
    API_DrvLockStatusConfigurationRegistersWP   = 0x20,
    API_DrvLockStatusConfigurationRegistersPermanent,
    API_DrvProtectSectorBlock,
    API_DrvProtectTopBottom,
    API_DrvSelectBlockProtectionRange,
    API_DrvSetIO3Reset,
    API_DrvSetOutputImpedance,
    API_DrvSelectWriteProtection,
    API_DrvSetReadLatency                   = 0x30,
    API_DrvDisableWrite,
    API_DrvEnableWrite,
    API_DrvEnableWriteVolatile,
    API_DrvSetBurstWrapLength,
    API_DrvEnableBurstWrap,
    API_DrvDisableBurstWrap,
    API_DrvEnableQPI,
    API_DrvDisableQPI,
    API_DrvReadDLP,
    API_DrvProgramDLP,
    API_DrvReadArray,
    API_DrvBeginContinuousReadArray,
    API_DrvContinueContinuousReadArray,
    API_DrvEndContinuousReadArray,
    API_DrvProgramArray,
    API_DrvProgramArrayBlankCheck,
    API_DrvEraseSector,
    API_DrvEraseHalfBlock,
    API_DrvEraseBlock,
    API_DrvEraseChip,
    API_DrvEraseChipAlternate,
    API_DrvSuspendErase,
    API_DrvResumeErase,
    API_DrvSuspendProgram,
    API_DrvResumeProgram,
    API_DrvSelectSuspendCommand,
    API_DrvSelectResumeCommand,
    API_DrvCheckSectorBlank,
    API_DrvEraseSecurityRegion              = 0x60,
    API_DrvProgramSecurityRegion,
    API_DrvReadSecurityRegion,
    API_DrvReadIBL,
    API_DrvLockIBL,
    API_DrvUnlockIBL,
    API_DrvLockGlobalIBL,
    API_DrvUnlockGlobalIBL,
    API_DrvSetPRPAddress,
    API_DrvGetPRPAddress,
    API_DrvSetPRPTopBottom,
    API_DrvGetPRPTopBottom,
    API_DrvEnablePRP,
    API_DrvDisablePRP,
    API_DrvSetPRPProtectedSectors,
    API_DrvGetPRPProtectedSectors,
    API_DrvSetIRPProtectionMode,
    API_DrvGetIRPProtectionMode,
    API_DrvSetIRPIBLLockBootBit,
    API_DrvGetIRPIBLLockBootBit,
    API_DrvSetSecurityRegion3ReadPasswordMode,
    API_DrvGetSecurityRegion3ReadPasswordMode,
    API_DrvGetSecurityRegion23PRPStatus,
    API_DrvLockSecurityRegion23PRP,
    API_DrvReadPassword                     = 0xD0,
    API_DrvProgramPassword,
    API_DrvSendPassword,
    API_DrvSWReset,
    API_DrvHWReset,
    API_DrvModeBitReset,
    API_DrvEnterDeepPowerDown,
    API_DrvReleaseDeepPowerDown
};



#define IOC_SYS                             'S'
#define IOC_DRV_GP                          'G'	// General Purpose
#define IOC_DRV_OP                          'O'	// Operating Purpose
#define IOC_DRV_DEVFUN                      'F'	// Device Function

#define IOCNR_SysSetGlobalVar                               API_SysSetGlobalVar
#define IOCNR_SysGetGlobalVar                               API_SysGetGlobalVar
#define IOCNR_SysFindDevice                                 API_SysFindDevice
#define IOCNR_SysCloseDevice                                API_SysCloseDevice
#define IOCNR_SysGetErrorText                               API_SysGetErrorText
#define IOCNR_SysCheckUserBreak                             API_SysCheckUserBreak
#define IOCNR_SysShowDriverInformation                      API_SysShowDriverInformation
#define IOCNR_SysWaitUSec                                   API_SysWaitUSec
#define IOCNR_SysFormatDeltaTime                            API_SysFormatDeltaTime
#define IOCNR_SysGetDeltaTime                               API_SysGetDeltaTime
#define IOCNR_SysGetTimestamp                               API_SysGetTimestamp
#define IOCNR_SysSetLog                                     API_SysSetLog
#define IOCNR_SysClearLog                                   API_SysClearLog
#define IOCNR_SysOutputLog                                  API_SysOutputLog
#define IOCNR_SysGetLog                                     API_SysGetLog

#define IOCNR_DrvGetDeviceInfo                              API_DrvGetDeviceInfo
#define IOCNR_DrvGetDeviceGeometry                          API_DrvGetDeviceGeometry
#define IOCNR_DrvIncreaseStatusPollingTime                  API_DrvIncreaseStatusPollingTime
#define IOCNR_DrvSetSectorEraseTimeout                      API_DrvSetSectorEraseTimeout
#define IOCNR_DrvSetChipEraseTimeout                        API_DrvSetChipEraseTimeout
#define IOCNR_DrvSetWordProgramTimeout                      API_DrvSetWordProgramTimeout
#define IOCNR_DrvSetBufferProgramTimeout                    API_DrvSetBufferProgramTimeout
#define IOCNR_DrvSetEraseSuspendTimeout                     API_DrvSetEraseSuspendTimeout
#define IOCNR_DrvSetProgramSuspendTimeout                   API_DrvSetProgramSuspendTimeout
#define IOCNR_DrvSetPPBLockUnlockTimeout                    API_DrvSetPPBLockUnlockTimeout
#define IOCNR_DrvSetCRCSuspendTimeout                       API_DrvSetCRCSuspendTimeout
#define IOCNR_DrvSetEvaluateErasStatusTimeout               API_DrvSetEvaluateErasStatusTimeout
#define IOCNR_DrvGetBadSector                               API_DrvGetBadSector
#define IOCNR_DrvSetBadSector                               API_DrvSetBadSector
#define IOCNR_DrvClearBadSector                             API_DrvClearBadSector
#define IOCNR_DrvClearAllBadSectors                         API_DrvClearAllBadSectors

#define IOCNR_DrvSetAddressMode                             API_DrvSetAddressMode
#define IOCNR_DrvSetReadCommandMode                         API_DrvSetReadCommandMode
#define IOCNR_DrvSetProgramCommandMode                      API_DrvSetProgramCommandMode
#define IOCNR_DrvSetClock                                   API_DrvSetClock
#define IOCNR_DrvSetClockNotLC                              API_DrvSetClockNotLC
#define IOCNR_DrvGetClock                                   API_DrvGetClock
#define IOCNR_DrvSetWP                                      API_DrvSetWP
#define IOCNR_DrvSetupHold                                  API_DrvSetupHold
#define IOCNR_DrvEnableHold                                 API_DrvEnableHold
#define IOCNR_DrvDisableHold                                API_DrvDisableHold
#define IOCNR_DrvWriteOpcode                                API_DrvWriteOpcode
#define IOCNR_DrvWriteAddress                               API_DrvWriteAddress
#define IOCNR_DrvWriteModeBit                               API_DrvWriteModeBit
#define IOCNR_DrvWriteCount                                 API_DrvWriteCount
#define IOCNR_DrvReadDataBit                                API_DrvReadDataBit
#define IOCNR_DrvWriteDataBit                               API_DrvWriteDataBit
#define IOCNR_DrvReadControllerRegister                     API_DrvReadControllerRegister
#define IOCNR_DrvWriteControllerRegister                    API_DrvWriteControllerRegister
#define IOCNR_DrvResetController                            API_DrvResetController
#define IOCNR_DrvGetDevicePowerStatus                       API_DrvGetDevicePowerStatus
#define IOCNR_DrvPowerOnDevice                              API_DrvPowerOnDevice
#define IOCNR_DrvPowerOffDevice                             API_DrvPowerOffDevice

#define IOCNR_DrvReadID                                     API_DrvReadID
#define IOCNR_DrvReadSFDP                                   API_DrvReadSFDP
#define IOCNR_DrvReadUniqueID                               API_DrvReadUniqueID
#define IOCNR_DrvGetDeviceStatus                            API_DrvGetDeviceStatus
#define IOCNR_DrvReadStatusRegister                         API_DrvReadStatusRegister
#define IOCNR_DrvWriteStatusRegister                        API_DrvWriteStatusRegister
#define IOCNR_DrvClearDeviceStatus                          API_DrvClearDeviceStatus
#define IOCNR_DrvClearDeviceStatusAlternate                 API_DrvClearDeviceStatusAlternate
#define IOCNR_DrvReadConfigurationRegister                  API_DrvReadConfigurationRegister
#define IOCNR_DrvWriteConfigurationRegister                 API_DrvWriteConfigurationRegister
#define IOCNR_DrvReadNVRegister                             API_DrvReadNVRegister
#define IOCNR_DrvReadVRegister                              API_DrvReadVRegister
#define IOCNR_DrvWriteNVRegister                            API_DrvWriteNVRegister
#define IOCNR_DrvWriteVRegister                             API_DrvWriteVRegister
#define IOCNR_DrvEnableQuad                                 API_DrvEnableQuad
#define IOCNR_DrvDisableQuad                                API_DrvDisableQuad
#define IOCNR_DrvLockStatusConfigurationRegistersWP         API_DrvLockStatusConfigurationRegistersWP
#define IOCNR_DrvLockStatusConfigurationRegistersPermanent  API_DrvLockStatusConfigurationRegistersPermanent
#define IOCNR_DrvProtectSectorBlock                         API_DrvProtectSectorBlock
#define IOCNR_DrvProtectTopBottom                           API_DrvProtectTopBottom
#define IOCNR_DrvSelectBlockProtectionRange                 API_DrvSelectBlockProtectionRange
#define IOCNR_DrvSetIO3Reset                                API_DrvSetIO3Reset
#define IOCNR_DrvSetOutputImpedance                         API_DrvSetOutputImpedance
#define IOCNR_DrvSelectWriteProtection                      API_DrvSelectWriteProtection
#define IOCNR_DrvSetReadLatency                             API_DrvSetReadLatency
#define IOCNR_DrvDisableWrite                               API_DrvDisableWrite
#define IOCNR_DrvEnableWrite                                API_DrvEnableWrite
#define IOCNR_DrvEnableWriteVolatile                        API_DrvEnableWriteVolatile
#define IOCNR_DrvSetBurstWrapLength                         API_DrvSetBurstWrapLength
#define IOCNR_DrvEnableBurstWrap                            API_DrvEnableBurstWrap
#define IOCNR_DrvDisableBurstWrap                           API_DrvDisableBurstWrap
#define IOCNR_DrvEnableQPI                                  API_DrvEnableQPI
#define IOCNR_DrvDisableQPI                                 API_DrvDisableQPI
#define IOCNR_DrvReadDLP                                    API_DrvReadDLP
#define IOCNR_DrvProgramDLP                                 API_DrvProgramDLP
#define IOCNR_DrvReadArray                                  API_DrvReadArray
#define IOCNR_DrvBeginContinuousReadArray                   API_DrvBeginContinuousReadArray
#define IOCNR_DrvContinueContinuousReadArray                API_DrvContinueContinuousReadArray
#define IOCNR_DrvEndContinuousReadArray                     API_DrvEndContinuousReadArray
#define IOCNR_DrvProgramArray                               API_DrvProgramArray
#define IOCNR_DrvProgramArrayBlankCheck                     API_DrvProgramArrayBlankCheck
#define IOCNR_DrvEraseSector                                API_DrvEraseSector
#define IOCNR_DrvEraseHalfBlock                             API_DrvEraseHalfBlock
#define IOCNR_DrvEraseBlock                                 API_DrvEraseBlock
#define IOCNR_DrvEraseChip                                  API_DrvEraseChip
#define IOCNR_DrvEraseChipAlternate                         API_DrvEraseChipAlternate
#define IOCNR_DrvSuspendErase                               API_DrvSuspendErase
#define IOCNR_DrvResumeErase                                API_DrvResumeErase
#define IOCNR_DrvSuspendProgram                             API_DrvSuspendProgram
#define IOCNR_DrvResumeProgram                              API_DrvResumeProgram
#define IOCNR_DrvSelectSuspendCommand                       API_DrvSelectSuspendCommand
#define IOCNR_DrvSelectResumeCommand                        API_DrvSelectResumeCommand
#define IOCNR_DrvCheckSectorBlank                           API_DrvCheckSectorBlank
#define IOCNR_DrvEraseSecurityRegion                        API_DrvEraseSecurityRegion
#define IOCNR_DrvProgramSecurityRegion                      API_DrvProgramSecurityRegion
#define IOCNR_DrvReadSecurityRegion                         API_DrvReadSecurityRegion
#define IOCNR_DrvReadIBL                                    API_DrvReadIBL
#define IOCNR_DrvLockIBL                                    API_DrvLockIBL
#define IOCNR_DrvUnlockIBL                                  API_DrvUnlockIBL
#define IOCNR_DrvLockGlobalIBL                              API_DrvLockGlobalIBL
#define IOCNR_DrvUnlockGlobalIBL                            API_DrvUnlockGlobalIBL
#define IOCNR_DrvSetPRPAddress                              API_DrvSetPRPAddress
#define IOCNR_DrvGetPRPAddress                              API_DrvGetPRPAddress
#define IOCNR_DrvSetPRPTopBottom                            API_DrvSetPRPTopBottom
#define IOCNR_DrvGetPRPTopBottom                            API_DrvGetPRPTopBottom
#define IOCNR_DrvEnablePRP                                  API_DrvEnablePRP
#define IOCNR_DrvDisablePRP                                 API_DrvDisablePRP
#define IOCNR_DrvSetPRPProtectedSectors                     API_DrvSetPRPProtectedSectors
#define IOCNR_DrvGetPRPProtectedSectors                     API_DrvGetPRPProtectedSectors
#define IOCNR_DrvSetIRPProtectionMode                        API_DrvSetIRPProtectionMode
#define IOCNR_DrvGetIRPProtectionMode                        API_DrvGetIRPProtectionMode
#define IOCNR_DrvSetIRPIBLLockBootBit                       API_DrvSetIRPIBLLockBootBit
#define IOCNR_DrvGetIRPIBLLockBootBit                       API_DrvGetIRPIBLLockBootBit
#define IOCNR_DrvSetSecurityRegion3ReadPasswordMode         API_DrvSetSecurityRegion3ReadPasswordMode
#define IOCNR_DrvGetSecurityRegion3ReadPasswordMode         API_DrvGetSecurityRegion3ReadPasswordMode
#define IOCNR_DrvGetSecurityRegion23PRPStatus               API_DrvGetSecurityRegion23PRPStatus
#define IOCNR_DrvLockSecurityRegion23PRP                    API_DrvLockSecurityRegion23PRP
#define IOCNR_DrvReadPassword                               API_DrvReadPassword
#define IOCNR_DrvProgramPassword                            API_DrvProgramPassword
#define IOCNR_DrvSendPassword                               API_DrvSendPassword
#define IOCNR_DrvSWReset                                    API_DrvSWReset
#define IOCNR_DrvHWReset                                    API_DrvHWReset
#define IOCNR_DrvModeBitReset                               API_DrvModeBitReset
#define IOCNR_DrvEnterDeepPowerDown                         API_DrvEnterDeepPowerDown
#define IOCNR_DrvReleaseDeepPowerDown                       API_DrvReleaseDeepPowerDown



#define SLT_SysSetGlobalVar(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysSetGlobalVar, len)
#define SLT_SysGetGlobalVar(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysGetGlobalVar, len)
#define SLT_SysFindDevice(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysFindDevice, len)
#define SLT_SysCloseDevice(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysCloseDevice, len)
#define SLT_SysGetErrorText(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysGetErrorText, len)
#define SLT_SysCheckUserBreak(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysCheckUserBreak, len)
#define SLT_SysShowDriverInformation(len)               _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysShowDriverInformation, len)
#define SLT_SysWaitUSec(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysWaitUSec, len)
#define SLT_SysFormatDeltaTime(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysFormatDeltaTime, len)
#define SLT_SysGetDeltaTime(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysGetDeltaTime, len)
#define SLT_SysGetTimestamp(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysGetTimestamp, len)
#define SLT_SysSetLog(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysSetLog, len)
#define SLT_SysClearLog(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysClearLog, len)
#define SLT_SysOutputLog(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysOutputLog, len)
#define SLT_SysGetLog(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_SYS, IOCNR_SysGetLog, len)


#define SLT_DrvGetDeviceInfo(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvGetDeviceInfo, len)
#define SLT_DrvGetDeviceGeometry(len)                       _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvGetDeviceGeometry, len)
#define SLT_DrvIncreaseStatusPollingTime(len)               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvIncreaseStatusPollingTime, len)
#define SLT_DrvSetSectorEraseTimeout(len)                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetSectorEraseTimeout , len)
#define SLT_DrvSetChipEraseTimeout(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetChipEraseTimeout, len)
#define SLT_DrvSetWordProgramTimeout(len)                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetWordProgramTimeout, len)
#define SLT_DrvSetBufferProgramTimeout(len)                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetBufferProgramTimeout, len)
#define SLT_DrvSetEraseSuspendTimeout(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetEraseSuspendTimeout, len)
#define SLT_DrvSetProgramSuspendTimeout(len)                _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetProgramSuspendTimeout, len)
#define SLT_DrvSetPPBLockUnlockTimeout(len)                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetPPBLockUnlockTimeout, len)
#define SLT_DrvSetCRCSuspendTimeout(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetCRCSuspendTimeout, len)
#define SLT_DrvSetEvaluateErasStatusTimeout(len)            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetEvaluateErasStatusTimeout, len)
#define SLT_DrvGetBadSector(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvGetBadSector, len)
#define SLT_DrvSetBadSector(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvSetBadSector, len)
#define SLT_DrvClearBadSector(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvClearBadSector, len)
#define SLT_DrvClearAllBadSectors(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_GP, IOCNR_DrvClearAllBadSectors, len)

#define SLT_DrvSetAddressMode(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetAddressMode, len)
#define SLT_DrvSetReadCommandMode(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetReadCommandMode, len)
#define SLT_DrvSetProgramCommandMode(len)                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetProgramCommandMode, len)
#define SLT_DrvSetClock(len)                                _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetClock, len)
#define SLT_DrvSetClockNotLC(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetClockNotLC, len)
#define SLT_DrvGetClock(len)                                _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvGetClock, len)
#define SLT_DrvSetWP(len)                                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetWP, len)
#define SLT_DrvSetupHold(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvSetupHold, len)
#define SLT_DrvEnableHold(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvEnableHold, len)
#define SLT_DrvDisableHold(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvDisableHold, len)
#define SLT_DrvWriteOpcode(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteOpcode, len)
#define SLT_DrvWriteAddress(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteAddress, len)
#define SLT_DrvWriteModeBit(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteModeBit, len)
#define SLT_DrvWriteCount(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteCount, len)
#define SLT_DrvReadDataBit(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvReadDataBit, len)
#define SLT_DrvWriteDataBit(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteDataBit, len)
#define SLT_DrvReadControllerRegister(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvReadControllerRegister, len)
#define SLT_DrvWriteControllerRegister(len)                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvWriteControllerRegister, len)
#define SLT_DrvResetController(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvResetController, len)
#define SLT_DrvGetDevicePowerStatus(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvGetDevicePowerStatus, len)
#define SLT_DrvPowerOnDevice(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvPowerOnDevice, len)
#define SLT_DrvPowerOffDevice(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_OP, IOCNR_DrvPowerOffDevice, len)


#define SLT_DrvReadID(len)                                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadID, len)
#define SLT_DrvReadSFDP(len)                                _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadSFDP, len)
#define SLT_DrvReadUniqueID(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadUniqueID, len)
#define SLT_DrvGetDeviceStatus(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetDeviceStatus, len)
#define SLT_DrvReadStatusRegister(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadStatusRegister, len)
#define SLT_DrvWriteStatusRegister(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvWriteStatusRegister, len)
#define SLT_DrvClearDeviceStatus(len)                       _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvClearDeviceStatus, len)
#define SLT_DrvClearDeviceStatusAlternate(len)              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvClearDeviceStatusAlternate, len)
#define SLT_DrvReadConfigurationRegister(len)               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadConfigurationRegister, len)
#define SLT_DrvWriteConfigurationRegister(len)              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvWriteConfigurationRegister, len)
#define SLT_DrvReadNVRegister(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadNVRegister, len)
#define SLT_DrvReadVRegister(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadVRegister, len)
#define SLT_DrvWriteNVRegister(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvWriteNVRegister, len)
#define SLT_DrvWriteVRegister(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvWriteVRegister, len)
#define SLT_DrvEnableQuad(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnableQuad, len)
#define SLT_DrvDisableQuad(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvDisableQuad, len)
#define SLT_DrvLockStatusConfigurationRegistersWP(len)      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvLockStatusConfigurationRegistersWP, len)
#define SLT_DrvLockStatusConfigurationRegistersPermanent(len)    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvLockStatusConfigurationRegistersPermanent, len)
#define SLT_DrvProtectSectorBlock(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProtectSectorBlock, len)
#define SLT_DrvProtectTopBottom(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProtectTopBottom, len)
#define SLT_DrvSelectBlockProtectionRange(len)              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSelectBlockProtectionRange, len)
#define SLT_DrvSetIO3Reset(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetIO3Reset, len)
#define SLT_DrvSetOutputImpedance(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetOutputImpedance, len)
#define SLT_DrvSelectWriteProtection(len)                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSelectWriteProtection, len)
#define SLT_DrvSetReadLatency(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetReadLatency, len)
#define SLT_DrvDisableWrite(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvDisableWrite, len)
#define SLT_DrvEnableWrite(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnableWrite, len)
#define SLT_DrvEnableWriteVolatile(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnableWriteVolatile, len)
#define SLT_DrvSetBurstWrapLength(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetBurstWrapLength, len)
#define SLT_DrvEnableBurstWrap(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnableBurstWrap, len)
#define SLT_DrvDisableBurstWrap(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvDisableBurstWrap, len)
#define SLT_DrvEnableQPI(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnableQPI, len)
#define SLT_DrvDisableQPI(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvDisableQPI, len)
#define SLT_DrvReadDLP(len)                                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadDLP, len)
#define SLT_DrvProgramDLP(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProgramDLP, len)
#define SLT_DrvReadArray(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadArray, len)
#define SLT_DrvBeginContinuousReadArray(len)                _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvBeginContinuousReadArray, len)
#define SLT_DrvContinueContinuousReadArray(len)             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvContinueContinuousReadArray, len)
#define SLT_DrvEndContinuousReadArray(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEndContinuousReadArray, len)
#define SLT_DrvProgramArray(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProgramArray, len)
#define SLT_DrvProgramArrayBlankCheck(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProgramArrayBlankCheck, len)
#define SLT_DrvEraseSector(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseSector, len)
#define SLT_DrvEraseHalfBlock(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseHalfBlock, len)
#define SLT_DrvEraseBlock(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseBlock, len)
#define SLT_DrvEraseChip(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseChip, len)
#define SLT_DrvEraseChipAlternate(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseChipAlternate, len)
#define SLT_DrvSuspendErase(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSuspendErase, len)
#define SLT_DrvResumeErase(len)                             _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvResumeErase, len)
#define SLT_DrvSuspendProgram(len)                          _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSuspendProgram, len)
#define SLT_DrvResumeProgram(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvResumeProgram, len)
#define SLT_DrvSelectSuspendCommand(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSelectSuspendCommand, len)
#define SLT_DrvSelectResumeCommand(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSelectResumeCommand, len)
#define SLT_DrvCheckSectorBlank(len)                        _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvCheckSectorBlank, len)
#define SLT_DrvEraseSecurityRegion(len)                     _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEraseSecurityRegion, len)
#define SLT_DrvProgramSecurityRegion(len)                   _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProgramSecurityRegion, len)
#define SLT_DrvReadSecurityRegion(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadSecurityRegion, len)
#define SLT_DrvReadIBL(len)                                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadIBL, len)
#define SLT_DrvLockIBL(len)                                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvLockIBL, len)
#define SLT_DrvUnlockIBL(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvUnlockIBL, len)
#define SLT_DrvLockGlobalIBL(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvLockGlobalIBL, len)
#define SLT_DrvUnlockGlobalIBL(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvUnlockGlobalIBL, len)
#define SLT_DrvSetPRPAddress(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetPRPAddress, len)
#define SLT_DrvGetPRPAddress(len)                           _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetPRPAddress, len)
#define SLT_DrvSetPRPTopBottom(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetPRPTopBottom, len)
#define SLT_DrvGetPRPTopBottom(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetPRPTopBottom, len)
#define SLT_DrvEnablePRP(len)                               _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnablePRP, len)
#define SLT_DrvDisablePRP(len)                              _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvDisablePRP, len)
#define SLT_DrvSetPRPProtectedSectors(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetPRPProtectedSectors, len)
#define SLT_DrvGetPRPProtectedSectors(len)                  _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetPRPProtectedSectors, len)
#define SLT_DrvSetIRPProtectionMode(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetIRPProtectionMode, len)
#define SLT_DrvGetIRPProtectionMode(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetIRPProtectionMode, len)
#define SLT_DrvSetIRPIBLLockBootBit(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetIRPIBLLockBootBit, len)
#define SLT_DrvGetIRPIBLLockBootBit(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetIRPIBLLockBootBit, len)
#define SLT_DrvSetSecurityRegion3ReadPasswordMode(len)      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSetSecurityRegion3ReadPasswordMode, len)
#define SLT_DrvGetSecurityRegion3ReadPasswordMode(len)      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetSecurityRegion3ReadPasswordMode, len)
#define SLT_DrvGetSecurityRegion23PRPStatus(len)            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvGetSecurityRegion23PRPStatus, len)
#define SLT_DrvLockSecurityRegion23PRP(len)                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvLockSecurityRegion23PRP, len)
#define SLT_DrvReadPassword(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReadPassword, len)
#define SLT_DrvProgramPassword(len)                         _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvProgramPassword, len)
#define SLT_DrvSendPassword(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSendPassword, len)
#define SLT_DrvSWReset(len)                                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvSWReset, len)
#define SLT_DrvHWReset(len)                                 _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvHWReset, len)
#define SLT_DrvModeBitReset(len)                            _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvModeBitReset, len)
#define SLT_DrvEnterDeepPowerDown(len)                      _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvEnterDeepPowerDown, len)
#define SLT_DrvReleaseDeepPowerDown(len)                    _IOC(_IOC_READ|_IOC_WRITE, IOC_DRV_DEVFUN, IOCNR_DrvReleaseDeepPowerDown, len)
/* public function prototypes */

// HAL initialization function
SLLD_STATUS FLASH_INIT
(
int sel,            /* flash type select */
BYTE * name  /* falsh name */
);

SLLD_STATUS FLASH_DEINIT
(
void
);

// HAL read functions API
SLLD_STATUS FLASH_READ
(
BYTE    device_num,                     /* device number */
BYTE    command,                                    /* write a single command byte to flash */
ADDRESS sys_addr,                                   /* system address to be used */
BYTE   *data_buffer,                                /* Pointer to the data buffer where to store the read data */
int     Number_Of_Read_Bytes                        /* number of bytes to be read */
);

// HAL write functions API
SLLD_STATUS FLASH_WRITE
(
BYTE    device_num,                     /* device number */
BYTE    command,                                    /* write a single command byte to flash */
ADDRESS sys_addr,                                   /* system address to be used */
BYTE   *data_buffer,                                /* Pointer to the data buffer containing data to be written */
int     Number_Of_Written_Bytes                     /* number of bytes to be written */
);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INC_lld_halh */

